@@ -76,8 +76,7 @@ var LibraryEmVal = {
 
   _emval_register__deps: ['$emval_free_list', '$emval_handle_array', '$init_emval'],
   _emval_register: function(value) {
-
-    switch(value){
+    switch (value) {
       case undefined :{ return 1; }
       case null :{ return 2; }
       case true :{ return 3; }
@@ -93,6 +92,7 @@ var LibraryEmVal = {
       }
   },
 
+  _emval_incref__sig: 'vi',
   _emval_incref__deps: ['$emval_handle_array'],
   _emval_incref: function(handle) {
     if (handle > 4) {
@@ -109,6 +109,7 @@ var LibraryEmVal = {
     }
   },
 
+  _emval_run_destructors__sig: 'vi',
   _emval_run_destructors__deps: ['_emval_decref', '$emval_handle_array', '$runDestructors'],
   _emval_run_destructors: function(handle) {
     var destructors = emval_handle_array[handle].value;
@@ -126,11 +127,13 @@ var LibraryEmVal = {
     return __emval_register({});
   },
 
+  _emval_new_cstring__sig: 'ii',
   _emval_new_cstring__deps: ['$getStringOrSymbol', '_emval_register'],
   _emval_new_cstring: function(v) {
     return __emval_register(getStringOrSymbol(v));
   },
 
+  _emval_take_value__sig: 'iii',
   _emval_take_value__deps: ['_emval_register', '$requireRegisteredType'],
   _emval_take_value: function(type, argv) {
     type = requireRegisteredType(type, '_emval_take_value');
@@ -166,14 +169,14 @@ var LibraryEmVal = {
     };
 #else
     var argsList = "";
-    for(var i = 0; i < argCount; ++i) {
+    for (var i = 0; i < argCount; ++i) {
         argsList += (i!==0?", ":"")+"arg"+i; // 'arg0, arg1, ..., argn'
     }
 
     var functionBody =
         "return function emval_allocator_"+argCount+"(constructor, argTypes, args) {\n";
 
-    for(var i = 0; i < argCount; ++i) {
+    for (var i = 0; i < argCount; ++i) {
         functionBody +=
             "var argType"+i+" = requireRegisteredType(Module['HEAP32'][(argTypes >>> 2) + "+i+"], \"parameter "+i+"\");\n" +
             "var arg"+i+" = argType"+i+".readValueFromPointer(args);\n" +
@@ -240,9 +243,10 @@ var LibraryEmVal = {
     })()('return this')();
   },
 #endif
+  _emval_get_global__sig: 'ii',
   _emval_get_global__deps: ['_emval_register', '$getStringOrSymbol', '$emval_get_global'],
   _emval_get_global: function(name) {
-    if(name===0){
+    if (name===0) {
       return __emval_register(emval_get_global());
     } else {
       name = getStringOrSymbol(name);
@@ -256,6 +260,7 @@ var LibraryEmVal = {
     return __emval_register(Module[name]);
   },
 
+  _emval_get_property__sig: 'iii',
   _emval_get_property__deps: ['_emval_register', '$requireHandle'],
   _emval_get_property: function(handle, key) {
     handle = requireHandle(handle);
@@ -263,6 +268,7 @@ var LibraryEmVal = {
     return __emval_register(handle[key]);
   },
 
+  _emval_set_property__sig: 'viii',
   _emval_set_property__deps: ['$requireHandle'],
   _emval_set_property: function(handle, key, value) {
     handle = requireHandle(handle);
@@ -270,7 +276,8 @@ var LibraryEmVal = {
     value = requireHandle(value);
     handle[key] = value;
   },
-
+    
+  _emval_as__sig: 'iiii',
   _emval_as__deps: ['_emval_register', '$requireHandle', '$requireRegisteredType'],
   _emval_as: function(handle, returnType, destructorsRef) {
     handle = requireHandle(handle);
@@ -281,6 +288,20 @@ var LibraryEmVal = {
     return returnType['toWireType'](destructors, handle);
   },
 
+  _emval_as_int64__deps: ['$requireHandle', '$requireRegisteredType'],
+  _emval_as_int64: function(handle, returnType, destructorsRef) {
+    handle = requireHandle(handle);
+    returnType = requireRegisteredType(returnType, 'emval::as');
+    return returnType['toWireType'](null, handle);
+  },
+
+  _emval_as_uint64__deps: ['$requireHandle', '$requireRegisteredType'],
+  _emval_as_uint64: function(handle, returnType, destructorsRef) {
+    handle = requireHandle(handle);
+    returnType = requireRegisteredType(returnType, 'emval::as');
+    return returnType['toWireType'](null, handle);
+  },
+
   _emval_equals__deps: ['$requireHandle'],
   _emval_equals: function(first, second) {
     first = requireHandle(first);
@@ -361,6 +382,7 @@ var LibraryEmVal = {
     return id;
   },
 
+  _emval_get_method_caller__sig: 'iii',
   _emval_get_method_caller__deps: ['_emval_addMethodCaller', '_emval_lookupTypes', '$new_', '$makeLegalFunctionName'],
   _emval_get_method_caller: function(argCount, argTypes) {
     var types = __emval_lookupTypes(argCount, argTypes);
@@ -436,6 +458,7 @@ var LibraryEmVal = {
     return caller(handle, methodName, __emval_allocateDestructors(destructorsRef), args);
   },
 
+  _emval_call_void_method__sig: 'viiii',
   _emval_call_void_method__deps: ['_emval_allocateDestructors', '$getStringOrSymbol', '$emval_methodCallers', '$requireHandle'],
   _emval_call_void_method: function(caller, handle, methodName, args) {
     caller = emval_methodCallers[caller];
