@@ -15,29 +15,58 @@ var addedLibraryItems = {};
 // Each such proxied function is identified via an ordinal number (this is not the same namespace as function pointers in general).
 var proxiedFunctionTable = ["null" /* Reserve index 0 for an undefined function*/];
 
-// Used internally. set when there is a main() function.
-// Also set when in a linkable module, as the main() function might
-// arrive from a dynamically-linked library, and not necessarily
-// the current compilation unit.
-// Also set for STANDALONE_WASM since the _start function is needed to call
-// static ctors, even if there is no user main.
-var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || STANDALONE_WASM;
-
 // Mangles the given C/JS side function name to assembly level function name (adds an underscore)
 function mangleCSymbolName(f) {
   return f[0] == '$' ? f.substr(1) : '_' + f;
 }
 
-// Reverses C/JS name mangling: _foo -> foo, and foo -> $foo.
-function demangleCSymbolName(f) {
-  return f[0] == '_' ? f.substr(1) : '$' + f;
+// Splits out items that pass filter. Returns also the original sans the filtered
+function splitter(array, filter) {
+  var splitOut = array.filter(filter);
+  var leftIn = array.filter((x) => !filter(x));
+  return { leftIn: leftIn, splitOut: splitOut };
+}
+
+// Functions that start with '$' should not be exported to the wasm module.
+// They are intended to be exclusive to JS code only.
+function isJsOnlyIdentifier(ident) {
+  return ident[0] == '$';
+}
+
+function escapeJSONKey(x) {
+  if (/^[\d\w_]+$/.exec(x) || x[0] === '"' || x[0] === "'") return x;
+  assert(!x.includes("'"), 'cannot have internal single quotes in keys: ' + x);
+  return "'" + x + "'";
+}
+
+function stringifyWithFunctions(obj) {
+  if (typeof obj === 'function') return obj.toString();
+  if (obj === null || typeof obj !== 'object') return JSON.stringify(obj);
+  if (Array.isArray(obj)) {
+    return '[' + obj.map(stringifyWithFunctions).join(',') + ']';
+  } else {
+    return '{' + keys(obj).map((key) => escapeJSONKey(key) + ':' + stringifyWithFunctions(obj[key])).join(',') + '}';
+  }
+}
+
+function isDefined(symName) {
+  if (symName in WASM_EXPORTS || symName in SIDE_MODULE_EXPORTS) {
+    return true;
+  }
+  // 'invoke_' symbols are created at runtime in libary_dylink.py so can
+  // always be considered as defined.
+  if (RELOCATABLE && symName.startsWith('invoke_')) {
+    return true;
+  }
+  return false;
 }
 
 // JSifier
-function JSify(data, functionsOnly) {
+function JSify(functionsOnly) {
   var mainPass = !functionsOnly;
+  var functionStubs = [];
 
-  var itemsDict = { type: [], GlobalVariableStub: [], functionStub: [], function: [], GlobalVariable: [], GlobalVariablePostSet: [] };
+  var itemsDict = { type: [], functionStub: [], function: [], globalVariablePostSet: [] };
 
   if (mainPass) {
     // Add additional necessary items for the main pass. We can now do this since types are parsed (types can be used through
@@ -56,9 +85,10 @@ function JSify(data, functionsOnly) {
     } else {
       libFuncsToInclude = DEFAULT_LIBRARY_FUNCS_TO_INCLUDE;
     }
-    libFuncsToInclude.forEach(function(ident) {
-      data.functionStubs.push({
-        ident: mangleCSymbolName(ident)
+    libFuncsToInclude.forEach((ident) => {
+      functionStubs.push({
+        identOrig: ident,
+        identMangled: mangleCSymbolName(ident)
       });
     });
   }
@@ -67,18 +97,22 @@ function JSify(data, functionsOnly) {
     // It is possible that when printing the function as a string on Windows, the js interpreter we are in returns the string with Windows
     // line endings \r\n. This is undesirable, since line endings are managed in the form \n in the output for binary file writes, so
     // make sure the endings are uniform.
-    snippet = snippet.toString().replace(/\r\n/gm,"\n");
+    snippet = snippet.toString().replace(/\r\n/gm,'\n');
 
     // name the function; overwrite if it's already named
     snippet = snippet.replace(/function(?:\s+([^(]+))?\s*\(/, 'function ' + finalName + '(');
 
     // apply LIBRARY_DEBUG if relevant
-    if (LIBRARY_DEBUG) {
-      snippet = modifyFunction(snippet, function(name, args, body) {
-        return 'function ' + name + '(' + args + ') {\n' +
-               'var ret = (function() { if (runtimeDebug) err("[library call:' + finalName + ': " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");\n' +
-                body +
-                '}).apply(this, arguments); if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret)); return ret; \n}\n';
+    if (LIBRARY_DEBUG && !isJsOnlyIdentifier(ident)) {
+      snippet = modifyFunction(snippet, (name, args, body) => {
+        return `\
+function ${name}(${args}) {
+  var ret = (function() { if (runtimeDebug) err("[library call:${finalName}: " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");
+  ${body}
+  }).apply(this, arguments);
+  if (runtimeDebug && typeof ret !== "undefined") err("  [     return:" + prettyPrint(ret));
+  return ret;
+}`
       });
     }
     return snippet;
@@ -89,51 +123,51 @@ function JSify(data, functionsOnly) {
     // In LLVM, exceptions generate a set of functions of form __cxa_find_matching_catch_1(), __cxa_find_matching_catch_2(), etc.
     // where the number specifies the number of arguments. In Emscripten, route all these to a single function '__cxa_find_matching_catch'
     // that variadically processes all of these functions using JS 'arguments' object.
-    if (item.ident.startsWith('___cxa_find_matching_catch_')) {
+    if (item.identMangled.startsWith('___cxa_find_matching_catch_')) {
       if (DISABLE_EXCEPTION_THROWING) {
         error('DISABLE_EXCEPTION_THROWING was set (likely due to -fno-exceptions), which means no C++ exception throwing support code is linked in, but exception catching code appears. Either do not set DISABLE_EXCEPTION_THROWING (if you do want exception throwing) or compile all source files with -fno-except (so that no exceptions support code is required); also make sure DISABLE_EXCEPTION_CATCHING is set to the right value - if you want exceptions, it should be off, and vice versa.');
         return;
       }
-      var num = +item.ident.split('_').slice(-1)[0];
+      var num = +item.identMangled.split('_').slice(-1)[0];
       addCxaCatch(num);
       // Continue, with the code below emitting the proper JavaScript based on
       // what we just added to the library.
     }
 
-    function addFromLibrary(ident, dependent) {
+    const TOP_LEVEL = 'top-level compiled C/C++ code';
+
+    function addFromLibrary(item, dependent) {
+      // dependencies can be JS functions, which we just run
+      if (typeof item == 'function') return item();
+
+      var ident = item.identOrig;
+      var finalName = item.identMangled;
+
       if (ident in addedLibraryItems) return '';
       addedLibraryItems[ident] = true;
 
-      // dependencies can be JS functions, which we just run
-      if (typeof ident == 'function') return ident();
-
       // don't process any special identifiers. These are looked up when processing the base name of the identifier.
       if (isJsLibraryConfigIdentifier(ident)) {
         return '';
       }
 
-      var finalName = mangleCSymbolName(ident);
-
-      // if the function was implemented in compiled code, we just need to export it so we can reach it from JS
-      if (finalName in IMPLEMENTED_FUNCTIONS) {
-        EXPORTED_FUNCTIONS[finalName] = 1;
-        // stop here: we don't need to add anything from our js libraries, not even deps, compiled code is on it
+      // if the function was implemented in compiled code, there is no need to include the js version
+      if (ident in WASM_EXPORTS) {
         return '';
       }
 
-      // Don't replace implemented functions with library ones (which can happen when we add dependencies).
-      // Note: We don't return the dependencies here. Be careful not to end up where this matters
-      if (finalName in Functions.implementedFunctions) return '';
-
       var noExport = false;
 
-      if (!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) {
-        if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
+      if (!LibraryManager.library.hasOwnProperty(ident)) {
+        if (ONLY_CALC_JS_SYMBOLS) {
+          return;
+        }
+        if (!isDefined(ident)) {
           var msg = 'undefined symbol: ' + ident;
-          if (dependent) msg += ' (referenced by ' + dependent + ')';
+          if (dependent) msg += ` (referenced by ${dependent})`;
           if (ERROR_ON_UNDEFINED_SYMBOLS) {
             error(msg);
-            if (!LLD_REPORT_UNDEFINED) {
+            if (dependent == TOP_LEVEL && !LLD_REPORT_UNDEFINED) {
               warnOnce('Link with `-s LLD_REPORT_UNDEFINED` to get more information on undefined symbols');
             }
             warnOnce('To disable errors for undefined symbols use `-s ERROR_ON_UNDEFINED_SYMBOLS=0`')
@@ -147,49 +181,41 @@ function JSify(data, functionsOnly) {
         }
         if (!RELOCATABLE) {
           // emit a stub that will fail at runtime
-          LibraryManager.library[ident] = new Function("err('missing function: " + ident + "'); abort(-1);");
+          LibraryManager.library[ident] = new Function(`err('missing function: ${ident}'); abort(-1);`);
           // We have already warned/errored about this function, so for the purposes of Closure use, mute all type checks
           // regarding this function, marking ot a variadic function that can take in anything and return anything.
           // (not useful to warn/error multiple times)
           LibraryManager.library[ident + '__docs'] = '/** @type {function(...*):?} */';
         } else {
-          var isGlobalAccessor = ident.startsWith('g$');
-          var realIdent = ident;
-          if (isGlobalAccessor) {
-            realIdent = realIdent.substr(2);
-          }
-
-          var target = "Module['" + mangleCSymbolName(realIdent) + "']";
+          var target = `Module['${finalName}']`;
           var assertion = '';
           if (ASSERTIONS) {
             var what = 'function';
-            if (isGlobalAccessor) {
-              what = 'global';
-            }
-            assertion += 'if (!' + target + ') abort("external ' + what + ' \'' + realIdent + '\' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n';
+            assertion += `if (!${target}) abort("external symbol '${ident}' is missing. perhaps a side module was not linked in? if this function was expected to arrive from a system library, try to build the MAIN_MODULE with EMCC_FORCE_STDLIBS=1 in the environment");\n`;
 
           }
-          var functionBody;
-          if (isGlobalAccessor) {
-            functionBody = assertion + "return " + target + ";"
-          } else {
-            functionBody = assertion + "return " + target + ".apply(null, arguments);";
-          }
+          var functionBody = assertion + `return ${target}.apply(null, arguments);`;
           LibraryManager.library[ident] = new Function(functionBody);
           noExport = true;
         }
       }
 
+      if (!ALLOW_UNIMPLEMENTED_SYSCALLS && LibraryManager.library[ident + '__unimplemented']) {
+        error(`attempt to link unsupport syscall: ${ident} (use -s ALLOW_UNIMPLEMENTED_SYSCALLS (the default) to allow linking with a stub version`);
+      }
+
       var original = LibraryManager.library[ident];
       var snippet = original;
       var redirectedIdent = null;
       var deps = LibraryManager.library[ident + '__deps'] || [];
       if (!Array.isArray(deps)) {
-        error('JS library directive ' + ident + '__deps=' + deps.toString() + ' is of type ' + typeof deps + ', but it should be an array!');
+        error(`JS library directive ${ident}__deps=${deps.toString()} is of type ${typeof deps}, but it should be an array!`);
         return;
       }
-      deps.forEach(function(dep) {
-        if (typeof snippet === 'string' && !(dep in LibraryManager.library)) warn('missing library dependency ' + dep + ', make sure you are compiling with the right options (see #ifdefs in src/library*.js)');
+      deps.forEach((dep) => {
+        if (typeof snippet === 'string' && !(dep in LibraryManager.library)) {
+          warn(`missing library dependency ${dep}, make sure you are compiling with the right options (see #if in src/library*.js)`);
+        }
       });
       var isFunction = false;
 
@@ -206,7 +232,7 @@ function JSify(data, functionsOnly) {
           // In asm, we need to know about library functions. If there is a target, though, then no
           // need to consider this a library function - we will call directly to it anyhow
           if (!redirectedIdent && (typeof target == 'function')) {
-            Functions.libraryFunctions[finalName] = 1;
+            libraryFunctions.push(finalName);
           }
         }
       } else if (typeof snippet === 'object') {
@@ -214,15 +240,13 @@ function JSify(data, functionsOnly) {
       } else if (typeof snippet === 'function') {
         isFunction = true;
         snippet = processLibraryFunction(snippet, ident, finalName);
-        if (!isJsOnlyIdentifier(ident[0])) {
-          Functions.libraryFunctions[finalName] = 1;
-        }
+        libraryFunctions.push(finalName);
       }
 
-      // If a JS library item specifies xxx_import: true, then explicitly mark that symbol to be imported
-      // to asm.js/wasm module.
+      // If a JS library item specifies xxx_import: true, then explicitly mark that symbol to be exported
+      // to wasm module.
       if (LibraryManager.library[ident + '__import']) {
-        Functions.libraryFunctions[finalName] = 1;
+        libraryFunctions.push(finalName);
       }
 
       if (ONLY_CALC_JS_SYMBOLS)
@@ -238,7 +262,7 @@ function JSify(data, functionsOnly) {
         }
         if (postset && !addedLibraryItems[postsetId]) {
           addedLibraryItems[postsetId] = true;
-          itemsDict.GlobalVariablePostSet.push({
+          itemsDict.globalVariablePostSet.push({
             JS: postset + ';'
           });
         }
@@ -247,41 +271,40 @@ function JSify(data, functionsOnly) {
       if (redirectedIdent) {
         deps = deps.concat(LibraryManager.library[redirectedIdent + '__deps'] || []);
       }
-      // In asm, dependencies implemented in C might be needed by JS library functions.
-      // We don't know yet if they are implemented in C or not. To be safe, export such
-      // special cases.
-      [LIBRARY_DEPS_TO_AUTOEXPORT].forEach(function(special) {
-        deps.forEach(function(dep) {
-          if (dep == special && !EXPORTED_FUNCTIONS[dep]) {
-            EXPORTED_FUNCTIONS[dep] = 1;
-          }
-        });
-      });
-      if (VERBOSE) printErr('adding ' + finalName + ' and deps ' + deps + ' : ' + (snippet + '').substr(0, 40));
+      if (VERBOSE) {
+        printErr(`adding ${finalName} and deps ${deps} : ` + (snippet + '').substr(0, 40));
+      }
       var identDependents = ident + "__deps: ['" + deps.join("','")+"']";
       function addDependency(dep) {
-        return addFromLibrary(dep, identDependents + ', referenced by ' + dependent);
+        if (typeof dep !== 'function') {
+          dep = {identOrig: dep, identMangled: mangleCSymbolName(dep)};
+        }
+        return addFromLibrary(dep, `${identDependents}, referenced by ${dependent}`);
       }
-      var depsText = (deps ? deps.map(addDependency).filter(function(x) { return x != '' }).join('\n') + '\n' : '');
+      var depsText = (deps ? deps.map(addDependency).filter((x) => x != '').join('\n') + '\n' : '');
       var contentText;
       if (isFunction) {
         // Emit the body of a JS library function.
         var proxyingMode = LibraryManager.library[ident + '__proxy'];
         if (USE_PTHREADS && proxyingMode) {
           if (proxyingMode !== 'sync' && proxyingMode !== 'async') {
-            throw 'Invalid proxyingMode ' + ident + '__proxy: \'' + proxyingMode + '\' specified!';
+            throw `Invalid proxyingMode ${ident}__proxy: '${proxyingMode}' specified!`;
           }
           var sync = proxyingMode === 'sync';
           assert(typeof original === 'function');
-          contentText = modifyFunction(snippet, function(name, args, body) {
-            return 'function ' + name + '(' + args + ') {\n' +
-                   'if (ENVIRONMENT_IS_PTHREAD) return _emscripten_proxy_to_main_thread_js(' + proxiedFunctionTable.length + ', ' + (+sync) + (args ? ', ' : '') + args + ');\n' + body + '}\n';
+          contentText = modifyFunction(snippet, (name, args, body) => {
+            return `
+function ${name}(${args}) {
+  if (ENVIRONMENT_IS_PTHREAD)
+    return _emscripten_proxy_to_main_thread_js(${proxiedFunctionTable.length}, ${+sync}${args ? ', ' : ''}${args});
+  ${body}
+}\n`;
           });
           proxiedFunctionTable.push(finalName);
         } else {
           contentText = snippet; // Regular JS function that will be executed in the context of the calling thread.
         }
-      } else if (typeof snippet === 'string' && snippet.indexOf(';') == 0) {
+      } else if (typeof snippet === 'string' && snippet.startsWith(';')) {
         // In JS libraries
         //   foo: ';[code here verbatim]'
         //  emits
@@ -294,13 +317,16 @@ function JSify(data, functionsOnly) {
         //  emits
         //   'var foo = [value];'
         if (typeof snippet === 'string' && snippet[0] == '=') snippet = snippet.substr(1);
-        contentText = 'var ' + finalName + '=' + snippet + ';';
+        contentText = `var ${finalName} = ${snippet};`;
       }
       var sig = LibraryManager.library[ident + '__sig'];
       // asm module exports are done in emscripten.py, after the asm module is ready. Here
       // we also export library methods as necessary.
       if ((EXPORT_ALL || (finalName in EXPORTED_FUNCTIONS)) && !noExport) {
-        contentText += '\nModule["' + finalName + '"] = ' + finalName + ';';
+        contentText += `\nModule["${finalName}"] = ${finalName};`;
+      }
+      if (MAIN_MODULE && sig) {
+        contentText += `\n${finalName}.sig = '${sig}';`;
       }
 
       var commentText = '';
@@ -312,15 +338,14 @@ function JSify(data, functionsOnly) {
     }
 
     itemsDict.functionStub.push(item);
-    var shortident = demangleCSymbolName(item.ident);
-    item.JS = addFromLibrary(shortident, 'top-level compiled C/C++ code');
+    item.JS = addFromLibrary(item, TOP_LEVEL);
   }
 
   // Final combiner
 
   function finalCombiner() {
-    var splitPostSets = splitter(itemsDict.GlobalVariablePostSet, function(x) { return x.ident && x.dependencies });
-    itemsDict.GlobalVariablePostSet = splitPostSets.leftIn;
+    var splitPostSets = splitter(itemsDict.globalVariablePostSet, (x) => x.ident && x.dependencies);
+    itemsDict.globalVariablePostSet = splitPostSets.leftIn;
     var orderedPostSets = splitPostSets.splitOut;
 
     var limit = orderedPostSets.length * orderedPostSets.length;
@@ -338,13 +363,13 @@ function JSify(data, functionsOnly) {
       }
     }
 
-    itemsDict.GlobalVariablePostSet = itemsDict.GlobalVariablePostSet.concat(orderedPostSets);
+    itemsDict.globalVariablePostSet = itemsDict.globalVariablePostSet.concat(orderedPostSets);
 
     //
 
     if (!mainPass) {
-      var generated = itemsDict.function.concat(itemsDict.type).concat(itemsDict.GlobalVariableStub).concat(itemsDict.GlobalVariable);
-      print(generated.map(function(item) { return item.JS; }).join('\n'));
+      var generated = itemsDict.function.concat(itemsDict.type);
+      print(generated.map((item) => item.JS).join('\n'));
       return;
     }
 
@@ -365,12 +390,10 @@ function JSify(data, functionsOnly) {
     var legalizedI64sDefault = legalizedI64s;
     legalizedI64s = false;
 
-    var globalsData = {functionStubs: []}
-    JSify(globalsData, true);
-    globalsData = null;
+    JSify(true);
 
-    var generated = itemsDict.functionStub.concat(itemsDict.GlobalVariablePostSet);
-    generated.forEach(function(item) { print(indentify(item.JS || '', 2)); });
+    var generated = itemsDict.functionStub.concat(itemsDict.globalVariablePostSet);
+    generated.forEach((item) => print(indentify(item.JS || '', 2)));
 
     legalizedI64s = legalizedI64sDefault;
 
@@ -385,7 +408,7 @@ function JSify(data, functionsOnly) {
       print(preprocess(read('arrayUtils.js')));
     }
 
-    if (SUPPORT_BASE64_EMBEDDING && !MINIMAL_RUNTIME) {
+    if ((SUPPORT_BASE64_EMBEDDING || FORCE_FILESYSTEM) && !MINIMAL_RUNTIME) {
       print(preprocess(read('base64Utils.js')));
     }
 
@@ -420,13 +443,18 @@ function JSify(data, functionsOnly) {
     var shellParts = read(shellFile).split('{{BODY}}');
     print(processMacros(preprocess(shellParts[1], shellFile)));
 
-    PassManager.serialize();
+    print('\n//FORWARDED_DATA:' + JSON.stringify({
+      libraryFunctions: libraryFunctions,
+      ATINITS: ATINITS.join('\n'),
+      ATMAINS: ATMAINS.join('\n'),
+      ATEXITS: ATEXITS.join('\n'),
+    }));
   }
 
   // Data
 
   if (mainPass) {
-    data.functionStubs.forEach(functionStubHandler);
+    functionStubs.forEach(functionStubHandler);
   }
 
   finalCombiner();
