@@ -79,8 +79,8 @@ var LibraryTracing = {
       EmscriptenTrace.loadWorkerViaXHR(collector_url + 'worker.js', function (worker) {
         EmscriptenTrace.worker = worker;
         EmscriptenTrace.worker.addEventListener('error', function (e) {
-          console.log('TRACE WORKER ERROR:');
-          console.log(e);
+          out('TRACE WORKER ERROR:');
+          out(e);
         }, false);
         EmscriptenTrace.worker.postMessage({ 'cmd': 'configure',
                                              'data_version': EmscriptenTrace.DATA_VERSION,
@@ -104,7 +104,7 @@ var LibraryTracing = {
       if (window && window['wtf']) {
         EmscriptenTrace.googleWTFEnabled = true;
       } else {
-        console.log('GOOGLE WTF NOT AVAILABLE TO ENABLE');
+        out('GOOGLE WTF NOT AVAILABLE TO ENABLE');
       }
     },
 
@@ -262,14 +262,11 @@ var LibraryTracing = {
   emscripten_trace_report_memory_layout: function() {
     if (EmscriptenTrace.postEnabled) {
       var memory_layout = {
-        'static_base':  STATIC_BASE,
-        'stack_base':   STACK_BASE,
-        'stack_top':    STACKTOP,
-        'stack_max':    STACK_MAX,
-#if !MINIMAL_RUNTIME
-        'dynamic_base': DYNAMIC_BASE,
-#endif
-        'dynamic_top':  HEAP32[DYNAMICTOP_PTR>>2],
+        'static_base':  {{{ GLOBAL_BASE }}},
+        'stack_base':   _emscripten_stack_get_base(),
+        'stack_top':    _emscripten_stack_get_current(),
+        'stack_max':    _emscripten_stack_get_end(),
+        'dynamic_top':  _sbrk(),
         'total_memory': HEAP8.length
       };
       var now = EmscriptenTrace.now();
