@@ -86,6 +86,7 @@ var LibraryManager = {
       libraries.push('library_strings.js');
     } else {
       libraries.push('library_browser.js');
+      libraries.push('library_wget.js');
     }
 
     if (USE_PTHREADS) { // TODO: Currently WebGL proxying makes pthreads library depend on WebGL.
@@ -105,7 +106,7 @@ var LibraryManager = {
 
       if (NODERAWFS) {
         // NODERAWFS requires NODEFS
-        if (!SYSTEM_JS_LIBRARIES.includes('library_nodefs.js')) {
+        if (!JS_LIBRARIES.includes('library_nodefs.js')) {
           libraries.push('library_nodefs.js');
         }
         libraries.push('library_noderawfs.js');
@@ -141,7 +142,7 @@ var LibraryManager = {
     }
 
     // Add any explicitly specified system JS libraries to link to, add those to link.
-    libraries = libraries.concat(SYSTEM_JS_LIBRARIES)
+    libraries = libraries.concat(JS_LIBRARIES)
 
     if (LZ4) {
       libraries.push('library_lz4.js');
@@ -416,6 +417,7 @@ function exportRuntime() {
     'setTempRet0',
     'callMain',
     'abort',
+    'keepRuntimeAlive',
   ];
 
   if (USE_OFFSET_CONVERTER) {
