@@ -99,37 +99,6 @@ Module['instantiateWasm'] = function(info, receiveInstance) {
 };
 #endif
 
-#if LOAD_SOURCE_MAP || USE_OFFSET_CONVERTER
-// When using postMessage to send an object, it is processed by the structured clone algorithm.
-// The prototype, and hence methods, on that object is then lost. This function adds back the lost prototype.
-// This does not work with nested objects that has prototypes, but it suffices for WasmSourceMap and WasmOffsetConverter.
-function resetPrototype(constructor, attrs) {
-  var object = Object.create(constructor.prototype);
-  for (var key in attrs) {
-    if (attrs.hasOwnProperty(key)) {
-      object[key] = attrs[key];
-    }
-  }
-  return object;
-}
-#endif
-
-#if LOAD_SOURCE_MAP
-var wasmSourceMapData;
-#endif
-#if USE_OFFSET_CONVERTER
-var wasmOffsetData, wasmOffsetConverter;
-#endif
-
-function moduleLoaded() {
-#if LOAD_SOURCE_MAP
-  wasmSourceMap = resetPrototype(Module['WasmSourceMap'], wasmSourceMapData);
-#endif
-#if USE_OFFSET_CONVERTER
-  wasmOffsetConverter = resetPrototype(Module['WasmOffsetConverter'], wasmOffsetData);
-#endif
-}
-
 self.onmessage = function(e) {
   try {
     if (e.data.cmd === 'load') { // Preload command that is called once per worker to parse and load the Emscripten code.
@@ -155,10 +124,10 @@ self.onmessage = function(e) {
       {{{ makeAsmImportsAccessInPthread('wasmMemory') }}} = e.data.wasmMemory;
 
 #if LOAD_SOURCE_MAP
-      wasmSourceMapData = e.data.wasmSourceMap;
+      Module['wasmSourceMapData'] = e.data.wasmSourceMap;
 #endif
 #if USE_OFFSET_CONVERTER
-      wasmOffsetData = e.data.wasmOffsetConverter;
+      Module['wasmOffsetData'] = e.data.wasmOffsetConverter;
 #endif
 
       {{{ makeAsmImportsAccessInPthread('buffer') }}} = {{{ makeAsmImportsAccessInPthread('wasmMemory') }}}.buffer;
@@ -172,7 +141,6 @@ self.onmessage = function(e) {
         return exports.default(Module);
       }).then(function(instance) {
         Module = instance;
-        moduleLoaded();
       });
 #else
       if (typeof e.data.urlOrBlob === 'string') {
@@ -186,24 +154,14 @@ self.onmessage = function(e) {
 #if MINIMAL_RUNTIME
       {{{ EXPORT_NAME }}}(imports).then(function (instance) {
         Module = instance;
-        moduleLoaded();
       });
 #else
       {{{ EXPORT_NAME }}}(Module).then(function (instance) {
         Module = instance;
-        moduleLoaded();
       });
 #endif
 #endif
-
-#if !MODULARIZE && !MINIMAL_RUNTIME
-      // MINIMAL_RUNTIME always compiled Wasm (&Wasm2JS) asynchronously, even in pthreads. But
-      // regular runtime and asm.js are loaded synchronously, so in those cases
-      // we are now loaded, and can post back to main thread.
-      moduleLoaded();
-#endif
-
-#endif
+#endif // MODULARIZE && EXPORT_ES6
     } else if (e.data.cmd === 'objectTransfer') {
       Module['PThread'].receiveObjectTransfer(e.data);
     } else if (e.data.cmd === 'run') {
@@ -266,24 +224,22 @@ self.onmessage = function(e) {
         // The thread might have finished without calling pthread_exit(). If so,
         // then perform the exit operation ourselves.
         // (This is a no-op if explicit pthread_exit() had been called prior.)
-        Module['PThread'].threadExit(result);
+        Module['__emscripten_thread_exit'](result);
 #else
         if (Module['keepRuntimeAlive']()) {
           Module['PThread'].setExitStatus(result);
         } else {
-          Module['PThread'].threadExit(result);
+          Module['__emscripten_thread_exit'](result);
         }
 #endif
       } catch(ex) {
-        if (ex === 'Canceled!') {
-          Module['PThread'].threadCancel();
-        } else if (ex != 'unwind') {
+        if (ex != 'unwind') {
 #if ASSERTIONS
           // FIXME(sbc): Figure out if this is still needed or useful.  Its not
           // clear to me how this check could ever fail.  In order to get into
           // this try/catch block at all we have already called bunch of
           // functions on `Module`.. why is this one special?
-          if (typeof(Module['_emscripten_futex_wake']) !== "function") {
+          if (typeof(Module['_emscripten_futex_wake']) !== 'function') {
             err("Thread Initialisation failed.");
             throw ex;
           }
@@ -297,28 +253,29 @@ self.onmessage = function(e) {
 #endif
             } else {
 #if ASSERTIONS
-              err('Pthread 0x' + Module['_pthread_self']().toString(16) + ' called exit(), calling threadExit.');
+              err('Pthread 0x' + Module['_pthread_self']().toString(16) + ' called exit(), calling _emscripten_thread_exit.');
 #endif
-              Module['PThread'].threadExit(ex.status);
+              Module['__emscripten_thread_exit'](ex.status);
             }
           }
           else
-#endif
+#endif // !MINIMAL_RUNTIME
           {
-            Module['PThread'].threadExit(-2);
+            Module['__emscripten_thread_exit'](-2);
             throw ex;
           }
 #if ASSERTIONS
         } else {
           // else e == 'unwind', and we should fall through here and keep the pthread alive for asynchronous events.
-          err('Pthread 0x' + Module['_pthread_self']().toString(16) + ' completed its pthread main entry point with an unwind, keeping the pthread worker alive for asynchronous operation.');
+          err('Pthread 0x' + Module['_pthread_self']().toString(16) + ' completed its main entry point with an `unwind`, keeping the worker alive for asynchronous operation.');
 #endif
         }
       }
     } else if (e.data.cmd === 'cancel') { // Main thread is asking for a pthread_cancel() on this thread.
       if (Module['_pthread_self']()) {
-        Module['PThread'].threadCancel();
+        Module['__emscripten_thread_exit'](-1/*PTHREAD_CANCELED*/);
       }
+      postMessage({ 'cmd': 'cancelDone' });
     } else if (e.data.target === 'setimmediate') {
       // no-op
     } else if (e.data.cmd === 'processThreadQueue') {
