@@ -5,10 +5,10 @@
  * found in the LICENSE file.
  */
 
+#include <assert.h>
 #include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
-// #include <emscripten/emscripten.h>
 #include <emscripten/threading.h>
 #include "../../system/lib/libc/musl/src/internal/pthread_impl.h"
 
@@ -26,24 +26,19 @@ locale_t do_test() {
   return loc;
 }
 
-void *thread_test(void *t) 
-{
+void *thread_test(void *t) {
   puts("Doing test in child thread");
   pthread_exit((void*)do_test());
 }
 
-int main (int argc, char *argv[])
-{
+int main (int argc, char *argv[]) {
   puts("Doing test in main thread");
   locale_t main_loc = do_test();
   locale_t child_loc;
 
-  if (!emscripten_has_threading_support())
-  {
+  if (!emscripten_has_threading_support()) {
     child_loc = main_loc;
-  }
-  else
-  {
+  } else {
     long id = 1;
     pthread_t thread;
 
@@ -52,10 +47,6 @@ int main (int argc, char *argv[])
     pthread_join(thread, (void**)&child_loc);
   }
 
-#ifdef REPORT_RESULT
-  int result = (main_loc == child_loc);
-  REPORT_RESULT(result);
-#endif
-
-  pthread_exit(NULL);
+  assert(main_loc == child_loc);
+  return 0;
 }
