@@ -8,6 +8,8 @@ import os
 TAG = 'version_4'
 HASH = '30a7b04652239bccff3cb1fa7cd8ae602791b5f502a96df39585c13ebc4bb2b64ba1598c0d1f5382028d94e04a5ca02185ea06bf7f4b3520f6df4cc253f9dd24'
 
+deps = ['sdl2']
+
 
 def needed(settings):
   return settings.USE_SDL_IMAGE == 2
@@ -24,9 +26,9 @@ def get(ports, settings, shared):
   libname = 'libSDL2_image'
   if formats != '':
     libname += '_' + formats
-  libname = ports.get_lib_name(libname)
+  libname += '.a'
 
-  def create():
+  def create(final):
     src_dir = os.path.join(ports.get_dir(), 'sdl2_image', 'SDL2_image-' + TAG)
     ports.install_headers(src_dir, target='SDL2')
     srcs = '''IMG.c IMG_bmp.c IMG_gif.c IMG_jpg.c IMG_lbm.c IMG_pcx.c IMG_png.c IMG_pnm.c IMG_tga.c
@@ -51,20 +53,16 @@ def get(ports, settings, shared):
       o_s.append(o)
     shared.safe_ensure_dirs(os.path.dirname(o_s[0]))
     ports.run_commands(commands)
-    final = os.path.join(ports.get_build_dir(), 'sdl2_image', libname)
     ports.create_lib(final, o_s)
-    return final
 
-  return [shared.Cache.get(libname, create, what='port')]
+  return [shared.Cache.get_lib(libname, create, what='port')]
 
 
 def clear(ports, settings, shared):
-  shared.Cache.get_path(ports.get_lib_name('libSDL2_image'))
+  shared.Cache.get_path('libSDL2_image.a')
 
 
 def process_dependencies(settings):
-  global deps
-  deps = ['sdl2']
   settings.USE_SDL = 2
   if 'png' in settings.SDL2_IMAGE_FORMATS:
     deps.append('libpng')
