@@ -5,6 +5,7 @@
 
 import os
 import shutil
+from pathlib import Path
 
 TAG = '1.2.11'
 HASH = 'a42b8359e76cf7b3ae70bf31f0f8a8caa407ac80e8fe08b838076cd5e45ac2e685dae45eb59db2d25543fb3b5bd13b843a02bb8373cda704d7238be50d5e9c68'
@@ -26,7 +27,7 @@ def get(ports, settings, shared):
     os.makedirs(dest_path)
     shutil.rmtree(dest_path, ignore_errors=True)
     shutil.copytree(source_path, dest_path)
-    open(os.path.join(dest_path, 'zconf.h'), 'w').write(zconf_h)
+    Path(dest_path, 'zconf.h').write_text(zconf_h)
     ports.install_headers(dest_path)
 
     # build
@@ -34,7 +35,7 @@ def get(ports, settings, shared):
     commands = []
     o_s = []
     for src in srcs:
-      o = os.path.join(ports.get_build_dir(), 'zlib', src + '.o')
+      o = os.path.join(dest_path, src + '.o')
       shared.safe_ensure_dirs(os.path.dirname(o))
       commands.append([shared.EMCC, os.path.join(dest_path, src), '-O2', '-o', o, '-I' + dest_path, '-w', '-c'])
       o_s.append(o)
